﻿Type=Class
Version=1.8
ModulesStructureVersion=1
B4i=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Public tv As TableView
	Private mCallback As Object
	Private mEventName As String
	Type CustomCell (Text As String, Children As List, Parent As CustomCell, _
		CCP As CustomCellPrivate, CustomView As Panel)
	Type CustomCellPrivate (Expanded As Boolean, Cell As TableCell, Level As Int)
	Private rootCell As CustomCell
	Private expand, collapse, noimage As Bitmap

End Sub

Public Sub Initialize (CallBack As Object, EventName As String)
	mEventName = EventName
	mCallback = CallBack
	tv.Initialize("tv", False)
	rootCell.Initialize
	rootCell.Children.Initialize
	rootCell.CCP.Initialize
	rootCell.CCP.Level = -1
	expand = LoadBitmap(File.DirAssets, "ExpandableTableView_expand.png")
	collapse = LoadBitmap(File.DirAssets, "ExpandableTableView_collapse.png")
	noimage = LoadBitmap(File.DirAssets, "ExpandableTableView_noimage.png")
End Sub

Public Sub getView As View
	Return tv
End Sub

Public Sub getRoot As CustomCell
	Return rootCell
End Sub

Public Sub RefreshItems
	CreateInternalCells(rootCell, False)
End Sub
Public Sub ExpandAll
	tv.Clear
	CreateInternalCells(rootCell, True)
	tv.ReloadAll
End Sub
Public Sub CollapseAll
	tv.Clear
	CreateInternalCells(rootCell, False)
	tv.ReloadAll
End Sub

Private Sub CreateInternalCells(Parent As CustomCell, ShouldExpandAll As Boolean)
	For Each c As CustomCell In Parent.Children
		c.Parent = Parent
		c.CCP.Initialize
		c.CCP.Level = Parent.CCP.Level + 1
		Dim tc As TableCell
		If c.CCP.Level = 0 OR ShouldExpandAll Then 
			tc = tv.AddSingleLine(c.Text)
		Else
			tc.InitializeSingleLine(c.Text)
		End If
		tc.Tag = c
		c.CCP.Cell = tc
		tc.IndentationLevel = c.CCP.Level
		tc.IndentationWidth = 20
		If c.CustomView.IsInitialized Then
			tc.CustomView = c.CustomView
		End If
		If c.Children.IsInitialized Then
			If ShouldExpandAll Then c.CCP.Expanded = True
			CreateInternalCells(c, ShouldExpandAll)
		End If
		SetCellBitmap(tc)
	Next
End Sub

Private Sub tv_SelectedChanged (SectionIndex As Int, cell As TableCell)
	Dim c As CustomCell = cell.Tag
	If c.Children.IsInitialized Then
		tv.BeginUpdates
		Dim StartIndex As Int = tv.GetItems(0).IndexOf(cell)
		If c.CCP.Expanded = False Then
			Dim NewCells As List
			NewCells.Initialize
			For Each ch As CustomCell In c.Children
				NewCells.Add(ch.CCP.cell)
				SetCellBitmap(ch.CCP.cell)
			Next
			tv.InsertCells(0, StartIndex + 1, NewCells)
			tv.EndUpdates
			tv.ScrollTo(0, StartIndex, tv.SCROLL_MIDDLE)
		Else
			For i = StartIndex + 1 To tv.GetItems(0).Size - 1
				Dim tc As TableCell = tv.GetItems(0).Get(i)
				Dim c2 As CustomCell = tc.Tag
				If c2.CCP.Level = c.CCP.Level Then Exit
				c2.CCP.Expanded = False
			Next
			'Log("StartIndex = " & i & ", Len = " & ( i - StartIndex - 1))
			tv.RemoveCells(0, StartIndex + 1, i - StartIndex - 1)	
			tv.EndUpdates
		End If
		c.CCP.Expanded = Not(c.CCP.Expanded)
		SetCellBitmap(cell)
		cell.Update
	End If
	CallSub2(mCallback, mEventName & "_SelectedChanged", c)
End Sub

Private Sub SetCellBitmap(tc As TableCell)
	Dim c As CustomCell = tc.Tag
	If c.Children.IsInitialized AND c.Children.Size > 0 Then
		If c.CCP.Expanded Then 
			tc.Bitmap = collapse
		Else
			tc.Bitmap = expand
		End If
	Else
		tc.Bitmap = noimage
	End If
End Sub 

